;(function() {
  var urlPrefix = 'js://invoke?';
  var nativeEventPrefix = 'appHostEvent_';
  var proxyUrlPrefix = 'js://proxyInvoke?';

  var appHost = function () {
    this.serialNo = 0;
    this.callbacks = {};
  };

  appHost.prototype.invoke = function(name, param, cb) {
    ++this.serialNo;
    var serialNo = this.serialNo;
    if (!!cb) {
      this.callbacks[serialNo] = cb;
    }

    var obj = {"name": name, "params": param, "serial":serialNo};
    var command = encodeURIComponent(JSON.stringify(obj));
    if(command) {
        window.location = urlPrefix + command;
    }
  };

  appHost.prototype.proxyInvoke = function(product, name, param, cb) {
    ++this.serialNo;
    var serialNo = this.serialNo;
    if (!!cb) {
      this.callbacks[serialNo] = cb;
    }

    var obj = {"product": product, "name": name, "params": param, "serial":serialNo};
    var command = encodeURIComponent(JSON.stringify(obj));
    if(command) {
        window.location = proxyUrlPrefix + command;
    }
  };

  appHost.prototype.onInvokeResult = function(serailNo, param) {
    var cb = this.callbacks[serailNo];
    if (!!!cb) {
        window.console.log("non-existed serail: " + serailNo);
        return;
    }
    delete this.callbacks[serailNo];
    cb(param);
  };

  appHost.prototype.onProxyInvokeResult = function(serailNo, param) {
    var cb = this.callbacks[serailNo];
    if (!!!cb) {
      window.console.log("non-existed serail: " + serailNo);
      return;
    }
      delete this.callbacks[serailNo];
      cb(param);
  };

  appHost.prototype.onNativeCall = function(obj) {
    if (obj) {
      var name = obj["name"];
      var params = obj["params"];
      if ("onInvokeResult" === name) {
        var id = params["serial"];
        delete params["serial"];
        this.onInvokeResult(id, params);
      } else if ("onProxyInvokeResult" === name) {
        var id = params["serial"];
        delete params["serial"];
        this.onProxyInvokeResult(id, params);
      }else if ("trigger" == name) {
        appHost.event.trigger(params["type"], params["detail"]);
      }
    }
  };

  var appHostEvent = function() {};

  appHostEvent.prototype.on = function(type, cb) {
    document.addEventListener(nativeEventPrefix + type, cb, false);
  };

  appHostEvent.prototype.trigger = function(type, detail) {
    var ev = document.createEvent('Event');
    ev.initEvent(nativeEventPrefix + type, true, true);
    ev.detail = detail;
    document.dispatchEvent(ev);
  };

  appHost.prototype.launchActivity = function (url) {
    this.invoke('launchActivity', {'url' : url});
  };

  appHost.prototype.log = function (msg) {
    this.invoke('log', {'msg' : msg});
  };

  if (!!!window.appHost) {
    var appHost = new appHost;
    appHost.event = new appHostEvent;
    window.appHost = appHost;
    if (!!!window.console || !!!window.console.log) {
      window.console.log = window.appHost.log;
    }
    if (!!window.appHostOnLoad) {
      window.appHostOnLoad();
    }
  }
})();